From: Greg KH <gregkh@suse.de>
To: linux-kernel@vger.kernel.org, stable@kernel.org
Cc: stable-review@kernel.org, torvalds@linux-foundation.org,
akpm@linux-foundation.org, alan@lxorguk.ukuu.org.uk,
Yinghai Lu <yinghai@kernel.org>,
Stefano Stabellini <stefano.stabellini@eu.citrix.com>,
"H. Peter Anvin" <hpa@zytor.com>
Subject: [32/35] x86: Cleanup highmap after brk is concluded
Date: Fri, 25 Mar 2011 17:04:04 -0700 [thread overview]
Message-ID: <20110326000459.189962134@clark.kroah.org> (raw)
In-Reply-To: <20110326000509.GA29736@kroah.com>
2.6.33-longterm review patch. If anyone has any objections, please let us know.
------------------
From: Yinghai Lu <yinghai@kernel.org>
commit e5f15b45ddf3afa2bbbb10c7ea34fb32b6de0a0e upstream.
Now cleanup_highmap actually is in two steps: one is early in head64.c
and only clears above _end; a second one is in init_memory_mapping() and
tries to clean from _brk_end to _end.
It should check if those boundaries are PMD_SIZE aligned but currently
does not.
Also init_memory_mapping() is called several times for numa or memory
hotplug, so we really should not handle initial kernel mappings there.
This patch moves cleanup_highmap() down after _brk_end is settled so
we can do everything in one step.
Also we honor max_pfn_mapped in the implementation of cleanup_highmap.
Signed-off-by: Yinghai Lu <yinghai@kernel.org>
Signed-off-by: Stefano Stabellini <stefano.stabellini@eu.citrix.com>
LKML-Reference: <alpine.DEB.2.00.1103171739050.3382@kaball-desktop>
Signed-off-by: H. Peter Anvin <hpa@zytor.com>
Signed-off-by: Greg Kroah-Hartman <gregkh@suse.de>
---
arch/x86/kernel/head64.c | 3 ---
arch/x86/kernel/setup.c | 5 +++++
arch/x86/mm/init.c | 19 -------------------
arch/x86/mm/init_64.c | 11 ++++++-----
4 files changed, 11 insertions(+), 27 deletions(-)
--- a/arch/x86/kernel/head64.c
+++ b/arch/x86/kernel/head64.c
@@ -76,9 +76,6 @@ void __init x86_64_start_kernel(char * r
/* Make NULL pointers segfault */
zap_identity_mappings();
- /* Cleanup the over mapped high alias */
- cleanup_highmap();
-
for (i = 0; i < NUM_EXCEPTION_VECTORS; i++) {
#ifdef CONFIG_EARLY_PRINTK
set_intr_gate(i, &early_idt_handlers[i]);
--- a/arch/x86/kernel/setup.c
+++ b/arch/x86/kernel/setup.c
@@ -295,6 +295,9 @@ static void __init init_gbpages(void)
static inline void init_gbpages(void)
{
}
+static void __init cleanup_highmap(void)
+{
+}
#endif
static void __init reserve_brk(void)
@@ -895,6 +898,8 @@ void __init setup_arch(char **cmdline_p)
reserve_brk();
+ cleanup_highmap();
+
/*
* Find and reserve possible boot-time SMP configuration:
*/
--- a/arch/x86/mm/init.c
+++ b/arch/x86/mm/init.c
@@ -283,25 +283,6 @@ unsigned long __init_refok init_memory_m
load_cr3(swapper_pg_dir);
#endif
-#ifdef CONFIG_X86_64
- if (!after_bootmem && !start) {
- pud_t *pud;
- pmd_t *pmd;
-
- mmu_cr4_features = read_cr4();
-
- /*
- * _brk_end cannot change anymore, but it and _end may be
- * located on different 2M pages. cleanup_highmap(), however,
- * can only consider _end when it runs, so destroy any
- * mappings beyond _brk_end here.
- */
- pud = pud_offset(pgd_offset_k(_brk_end), _brk_end);
- pmd = pmd_offset(pud, _brk_end - 1);
- while (++pmd <= pmd_offset(pud, (unsigned long)_end - 1))
- pmd_clear(pmd);
- }
-#endif
__flush_tlb_all();
if (!after_bootmem && e820_table_end > e820_table_start)
--- a/arch/x86/mm/init_64.c
+++ b/arch/x86/mm/init_64.c
@@ -49,6 +49,7 @@
#include <asm/numa.h>
#include <asm/cacheflush.h>
#include <asm/init.h>
+#include <asm/setup.h>
#include <linux/bootmem.h>
static unsigned long dma_reserve __initdata;
@@ -257,18 +258,18 @@ void __init init_extra_mapping_uc(unsign
* to the compile time generated pmds. This results in invalid pmds up
* to the point where we hit the physaddr 0 mapping.
*
* We limit the mappings to the region from _text to _end. _end is
* rounded up to the 2MB boundary. This catches the invalid pmds as
* well, as they are located before _text:
*/
void __init cleanup_highmap(void)
{
unsigned long vaddr = __START_KERNEL_map;
- unsigned long end = roundup((unsigned long)_end, PMD_SIZE) - 1;
+ unsigned long vaddr_end = __START_KERNEL_map + (max_pfn_mapped << PAGE_SHIFT);
+ unsigned long end = roundup((unsigned long)_brk_end, PMD_SIZE) - 1;
pmd_t *pmd = level2_kernel_pgt;
- pmd_t *last_pmd = pmd + PTRS_PER_PMD;
- for (; pmd < last_pmd; pmd++, vaddr += PMD_SIZE) {
+ for (; vaddr + PMD_SIZE - 1 < vaddr_end; pmd++, vaddr += PMD_SIZE) {
if (pmd_none(*pmd))
continue;
if (vaddr < (unsigned long) _text || vaddr > end)
next prev parent reply other threads:[~2011-03-26 0:06 UTC|newest]
Thread overview: 40+ messages / expand[flat|nested] mbox.gz Atom feed top
2011-03-26 0:05 [00/35] 2.6.33.9-longterm review Greg KH
2011-03-26 0:03 ` [01/35] smp_call_function_many: handle concurrent clearing of mask Greg KH
2011-03-26 0:03 ` [02/35] [PARISC] fix per-cpu flag problem in the cpu affinity checkers Greg KH
2011-03-26 0:03 ` [03/35] i2c: Fix typo in instantiating-devices document Greg KH
2011-03-26 0:03 ` [04/35] mmc: sdio: remember new card RCA when redetecting card Greg KH
2011-03-26 0:03 ` [05/35] powerpc/kdump: Fix race in kdump shutdown Greg KH
2011-03-30 23:27 ` Paul Gortmaker
2011-04-11 22:57 ` [stable] " Greg KH
2011-03-26 0:03 ` [06/35] powerpc: rtas_flash needs to use rtas_data_buf Greg KH
2011-03-26 0:03 ` [07/35] x86, binutils, xen: Fix another wrong size directive Greg KH
2011-03-26 0:03 ` [08/35] hwmon: (sht15) Fix integer overflow in humidity calculation Greg KH
2011-03-26 0:03 ` [09/35] ALSA: hda - VIA: Fix stereo mixer recording no sound issue Greg KH
2011-03-26 0:03 ` [10/35] ALSA: hda - VIA: Add missing support for VT1718S in A-A path Greg KH
2011-03-26 0:03 ` [11/35] aio: wake all waiters when destroying ctx Greg KH
2011-03-26 0:03 ` [12/35] shmem: let shared anonymous be nonlinear again Greg KH
2011-03-26 0:03 ` [13/35] PCI hotplug: acpiphp: set current_state to D0 in register_slot Greg KH
2011-03-26 0:03 ` [14/35] xen: set max_pfn_mapped to the last pfn mapped Greg KH
2011-03-26 0:03 ` [15/35] PCI: return correct value when writing to the "reset" attribute Greg KH
2011-03-26 0:03 ` [16/35] [PATCH] Revert "intel_idle: PCI quirk to prevent Lenovo Ideapad s10-3 boot hang" Greg KH
2011-03-26 0:03 ` [17/35] Prevent rt_sigqueueinfo and rt_tgsigqueueinfo from spoofing the signal code Greg KH
2011-03-26 0:03 ` [18/35] ext3: skip orphan cleanup on rocompat fs Greg KH
2011-03-26 0:03 ` [19/35] procfs: fix /proc/<pid>/maps heap check Greg KH
2011-03-26 0:03 ` [20/35] proc: protect mm start_code/end_code in /proc/pid/stat Greg KH
2011-03-26 0:03 ` [21/35] fbcon: Bugfix soft cursor detection in Tile Blitting Greg KH
2011-03-26 0:03 ` [22/35] nfsd41: modify the members value of nfsd4_op_flags Greg KH
2011-03-26 0:03 ` [23/35] nfsd: wrong index used in inner loop Greg KH
2011-03-26 0:03 ` [24/35] [media] uvcvideo: Fix uvc_fixup_video_ctrl() format search Greg KH
2011-03-26 0:03 ` [25/35] [media] uvcvideo: Fix descriptor parsing for video output devices Greg KH
2011-03-26 0:03 ` [26/35] ehci-hcd: Bug fix: dont set a QHs Halt bit Greg KH
2011-03-26 0:03 ` [27/35] USB: uss720 fixup refcount position Greg KH
2011-03-26 0:04 ` [28/35] USB: cdc-acm: fix memory corruption / panic Greg KH
2011-03-26 0:04 ` [29/35] USB: cdc-acm: fix potential null-pointer dereference Greg KH
2011-03-26 0:04 ` [30/35] USB: cdc-acm: fix potential null-pointer dereference on disconnect Greg KH
2011-03-26 0:04 ` [31/35] Input: xen-kbdfront - advertise either absolute or relative coordinates Greg KH
2011-03-26 0:04 ` Greg KH [this message]
2011-03-26 0:04 ` [33/35] SUNRPC: Never reuse the socket port after an xs_close() Greg KH
2011-03-26 0:04 ` [34/35] fs: call security_d_instantiate in d_obtain_alias V2 Greg KH
2011-03-26 0:24 ` Casey Schaufler
2011-03-26 16:11 ` Josef Bacik
2011-03-26 0:04 ` [35/35] dcdbas: force SMI to happen when expected Greg KH
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20110326000459.189962134@clark.kroah.org \
--to=gregkh@suse.de \
--cc=akpm@linux-foundation.org \
--cc=alan@lxorguk.ukuu.org.uk \
--cc=hpa@zytor.com \
--cc=linux-kernel@vger.kernel.org \
--cc=stable-review@kernel.org \
--cc=stable@kernel.org \
--cc=stefano.stabellini@eu.citrix.com \
--cc=torvalds@linux-foundation.org \
--cc=yinghai@kernel.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -257,18 +258,18 @@ void __init init_extra_mapping_uc(unsign
* to the point where we hit the physaddr 0 mapping.
*
- * We limit the mappings to the region from _text to _end. _end is
- * rounded up to the 2MB boundary. This catches the invalid pmds as
+ * We limit the mappings to the region from _text to _brk_end. _brk_end
+ * is rounded up to the 2MB boundary. This catches the invalid pmds as
* well, as they are located before _text:
*/
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®